/* Include la definizione della strutura AudioFx ed i prototipi
   delle funzioni che utilizzeremo nel nostro effetto.             */
#include "audiofx.h"
#include <assert.h>
#include <windows.h>

/* ------------- CreateEffect: crea l'effetto ------------------------ */
struct AudioFx* CreateEffect (audioMasterCallback amc)
{
 Program* pg;
 int k;
 struct AudioFx* fx = malloc(sizeof(struct AudioFx));
 if (!fx) return NULL; // Out of memory.

 
 /* Impostare qui gli attributi (flags) dell'effetto */
 fx->flags = effFlagsCanMono; //|effFlagsCanReplacing;

 /* Identificare l'effetto con una serie unica di 4 caratteri */
 fx->uniqueID = makefour('R','v','b','2');

 /* Stabilire se l'effetto accetta input mono (1) o stereo (2), o entrambi (2 con effFlagsCanMono) */
 fx->numInputs = 2;

 /* Uscita mono (1) o stereo (2) */
 fx->numOutputs = 2;

 /* Numero di parametri */
 fx->numParams = kNumParam;

 /* Numero di programmi (1) */
 fx->numPrograms = kNumProgram;

 /* Il programma corrente  il primo */
 fx->nCurProgram = 0;

 /* Altri parametri che di solito non hanno bisogno di essere modificati */
 fx->master = amc;
 fx->blockSize = 1024;
 fx->sampleRate = 44100.0F;
 fx->dispatcher = &MessageDispatcher;
 memset (fx->future, 0, 60);
 fx->getParameter = &GetParameter;
 fx->initialDelay = 0;
 fx->ioRatio = 0;
 fx->magic = kEffectMagic;
 fx->object = (void*)fx;
 fx->offQualities = 0;
 fx->process = &Process;
 fx->processReplacing = &ProcessReplacing;
 fx->realQualities = 0;
 fx->resvd1 = fx->resvd2 = 0;
 fx->setParameter = &SetParameter;
 fx->user = 0;
 fx->version = 1;

 /* Inizializzare qui i dati relativi all'effetto! 
    Notate che assumiamo una frequenza di campionamento pari a 44100Hz. */

 /* Imposta il primo programma */
 pg = &fx->programs[fx->nCurProgram];
 pg->dry = 0.7F;
 pg->wet = 0.3F;
 pg->room_size = 0.3F;
 pg->feedback = 0.3F;

 strcpy (pg->name, "Init");

 /* Copia il primo programma su tutti gli altri */
 for (k = 1; k < kNumProgram; ++k)
 {
	 fx->programs[k] = fx->programs[0];
 }

 /* azzera i puntatori ai buffer */
 fx->dly_left[0].samples = 0;
 fx->dly_left[1].samples = 0;
 fx->dly_left[2].samples = 0;
 fx->dly_left[3].samples = 0;
 
 fx->dly_right[0].samples = 0;
 fx->dly_right[1].samples = 0;
 fx->dly_right[2].samples = 0;
 fx->dly_right[3].samples = 0;
 
 fx->aps_left[0].samples = 0;
 fx->aps_left[1].samples = 0;
 
 fx->aps_right[0].samples = 0;
 fx->aps_right[1].samples = 0; 

 SetSampleRate(fx, 44100);
 update(fx);
 Suspend(fx);
 
 return fx;
}

void DestroyEffect (struct AudioFx* fx)
{
	if (fx)
	{
		/* libera i buffer */
		if (fx->dly_left[0].samples) free(fx->dly_left[0].samples);
		if (fx->dly_left[1].samples) free(fx->dly_left[1].samples);
		if (fx->dly_left[2].samples) free(fx->dly_left[2].samples);
		if (fx->dly_left[3].samples) free(fx->dly_left[3].samples);
			
		if (fx->dly_right[0].samples) free(fx->dly_right[0].samples);
		if (fx->dly_right[1].samples) free(fx->dly_right[1].samples);
		if (fx->dly_right[2].samples) free(fx->dly_right[2].samples);
		if (fx->dly_right[3].samples) free(fx->dly_right[3].samples);
		
		if (fx->aps_left[0].samples) free(fx->aps_left[0].samples);
		if (fx->aps_left[1].samples) free(fx->aps_left[1].samples);
		
		if (fx->aps_right[0].samples) free(fx->aps_right[0].samples);
		if (fx->aps_right[1].samples) free(fx->aps_right[1].samples);
		
		/* elimina la struttura effetto */
		free(fx);	 
	}
}

/* dispatcher */
long MessageDispatcher (struct AudioFx* fx,
						long message,
						long index,
						long value,
						void *ptr,
						float opt)
{
	long lRet = 0; /* Valore di ritorno */

	switch (message)
	{
		case effSetProgram:
			if (value < fx->numPrograms)
				SetProgram(fx, value);
		break;
		case effGetProgram: lRet = fx->nCurProgram; break;
		case effSetProgramName: SetProgramName (fx, (char*)ptr); break;
		case effGetProgramName: GetProgramName (fx, (char*)ptr); break;
		case effGetParamLabel: GetParameterLabel (fx, index, (char*)ptr); break;
        case effGetParamDisplay: GetParameterDisplay (fx, index, (char*)ptr); break;
        case effGetParamName: GetParameterName (fx, index, (char*)ptr); break;

		case effSetSampleRate: SetSampleRate (fx, opt); break;
		case effSetBlockSize:  SetBlockSize (fx, value); break;
		case effMainsChanged: if (!value) Suspend(fx); break;
		case effGetVu: lRet = (long)(GetVu(fx)*32767.0F); break;
		case effIdentify: lRet = makefour('N', 'v', 'E', 'f'); break;
		case effGetChunk: lRet = GetChunk (fx, (void**)ptr, index); break;
		case effSetChunk: lRet = SetChunk (fx, ptr, value, index); break;

		default: break;
	}
	return lRet;
}


Sample GetVu (FX)
{
	return 0.0F;
}

long GetChunk (FX, void**ppData, long isPreset)
{
	return 0L;
}

long SetChunk (FX, void*pData, long byteSize, long isPreset)
{
	return 0L;
}

void SetBlockSize (FX, long blockSize)
{
	fx->blockSize = blockSize;
}