#include "audiofx.h"

void imposta_delay (struct Delay_tag* delay, float ms, float fb, float sr, float level)
{
	int nsamples = ms * sr / 1000.0F;
	delay->prw = delay->samples;

	/* imposta feedback */
	delay->feedback = fb;

	/* imposta lunghezza delay */
	delay->pmax = delay->samples + nsamples;

	/* azzera buffer di delay */
	memset(delay->samples, 0, nsamples*sizeof(float));

	/* imposta livello */
	delay->level = level;	
}

void update (struct AudioFx* fx)
{
	int k;
	int size;
	float ms_delay;
	float decorr = 1.02; /* decorrelazione sinistro/destro */

	struct Program_tag* pg = fx->programs + fx->nCurProgram;

	/* delay dei filtri comb */

	/* 1 delay, da 20 a 35 ms... */
	ms_delay = 20 + pg->room_size * 15;
	imposta_delay(fx->dly_left, ms_delay, 0.9F * pg->feedback, fx->sampleRate, 0.24F);
	imposta_delay(fx->dly_right, ms_delay * decorr, 0.9F * pg->feedback, fx->sampleRate, 0.24F);

	/* 2 delay, da 30 a 55 ms... */
	ms_delay = 30 + pg->room_size * 25;
	imposta_delay(fx->dly_left + 1, ms_delay * decorr, 0.8F * pg->feedback, fx->sampleRate, 0.3F);
	imposta_delay(fx->dly_right + 1, ms_delay, 0.8F * pg->feedback, fx->sampleRate, 0.3F);

	/* 3 delay, da 47 a 80 ms... */
	ms_delay = 47 + pg->room_size * 33;
	imposta_delay(fx->dly_left + 2, ms_delay, 0.7F * pg->feedback, fx->sampleRate, 0.31F);
	imposta_delay(fx->dly_right + 2, ms_delay * decorr, 0.7F * pg->feedback, fx->sampleRate, 0.31F);

	/* 4 delay, da 80 a 120 ms... */
	ms_delay = 80 + pg->room_size * 120;
	imposta_delay(fx->dly_left + 3, ms_delay * decorr, 0.6F * pg->feedback, fx->sampleRate, 0.35F);
	imposta_delay(fx->dly_right + 3, ms_delay, 0.6F * pg->feedback, fx->sampleRate, 0.35F);

	/* allpass 1 */
	imposta_delay(fx->aps_left, 13 * decorr, 0.95F, fx->sampleRate, 1);
	imposta_delay(fx->aps_right, 13, 0.95F, fx->sampleRate, 1);

	/* allpass 2 */
	imposta_delay(fx->aps_left + 1, 7, 0.97F, fx->sampleRate, 1);
	imposta_delay(fx->aps_right + 1, 7 * decorr, 0.97F, fx->sampleRate, 1);
	
	fx->dry = pg->dry;
	fx->wet = pg->wet;
}



/* Imposta il valore del parametro nParameter. */
void SetParameter (struct AudioFx* fx, long nParameter, float value)
{
	struct Program_tag *pg = &fx->programs[fx->nCurProgram];

	switch (nParameter)
	{
		case kRoomSize: pg->room_size = value; break;
		case kFeedback: pg->feedback = value; break;
		case kWet: pg->wet = value; break;
		case kDry: pg->dry = value; break;
	}

	update(fx); /* Aggiorna! */
}

/* Imposta il numero di programma corrente. */
void SetProgram (struct AudioFx* fx, long nProgram)
{
	fx->nCurProgram = nProgram;
	update(fx);
}

/* Imposta il nome del programma corrente. */
void SetProgramName (struct AudioFx* fx, char* szProgramName)
{
	strcpy(fx->programs[fx->nCurProgram].name, szProgramName);
}

/* Imposta la frequenza di campionamento */
void SetSampleRate (struct AudioFx* fx, float sampleRate)
{
	int samples = sampleRate / 4;

	if (sampleRate != fx->sampleRate || fx->dly_left[0].samples == 0)
	{
		/* libera i buffer */
		if (fx->dly_left[0].samples) free(fx->dly_left[0].samples);
		if (fx->dly_left[1].samples) free(fx->dly_left[1].samples);
		if (fx->dly_left[2].samples) free(fx->dly_left[2].samples);
		if (fx->dly_left[3].samples) free(fx->dly_left[3].samples);

		if (fx->dly_right[0].samples) free(fx->dly_right[0].samples);
		if (fx->dly_right[1].samples) free(fx->dly_right[1].samples);
		if (fx->dly_right[2].samples) free(fx->dly_right[2].samples);
		if (fx->dly_right[3].samples) free(fx->dly_right[3].samples);

		if (fx->aps_left[0].samples) free(fx->aps_left[0].samples);
		if (fx->aps_left[1].samples) free(fx->aps_left[1].samples);
		
		if (fx->aps_right[0].samples) free(fx->aps_right[0].samples);
		if (fx->aps_right[1].samples) free(fx->aps_right[1].samples);

		/* alloca i buffer */
		fx->dly_left[0].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_left[0].samples, 0, samples*sizeof(float));

		fx->dly_left[1].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_left[1].samples, 0, samples*sizeof(float));

		fx->dly_left[2].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_left[2].samples, 0, samples*sizeof(float));

		fx->dly_left[3].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_left[3].samples, 0, samples*sizeof(float));


		fx->dly_right[0].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_right[0].samples, 0, samples*sizeof(float));
		
		fx->dly_right[1].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_right[1].samples, 0, samples*sizeof(float));
		
		fx->dly_right[2].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_right[2].samples, 0, samples*sizeof(float));
		
		fx->dly_right[3].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_right[3].samples, 0, samples*sizeof(float));


		fx->aps_left[0].samples = malloc (sizeof(float)*samples);
		memset(fx->aps_left[0].samples, 0, samples*sizeof(float));
		
		fx->aps_left[1].samples = malloc (sizeof(float)*samples);
		memset(fx->aps_left[1].samples, 0, samples*sizeof(float));

		fx->aps_right[0].samples = malloc (sizeof(float)*samples);
		memset(fx->aps_right[0].samples, 0, samples*sizeof(float));
		
		fx->aps_right[1].samples = malloc (sizeof(float)*samples);
		memset(fx->aps_right[1].samples, 0, samples*sizeof(float));		

		/* aggiorna l'effetto */
		update(fx);		
	}
}

/* Chiamata quando l'effetto viene spento */
void Suspend (struct AudioFx* fx)
{
	/* Azzera le linee di ritardo! */
	int k;
	for (k = 0; k < 4; ++k)
	{	
		memset (fx->dly_left[k].samples, 0, (int)(sizeof(float)*fx->sampleRate/4));
		memset (fx->dly_right[k].samples, 0, (int)(sizeof(float)*fx->sampleRate/4));
	}
}

/* Chiamata quando l'effetto viene acceso */
void Resume (struct AudioFx* fx)
{
}
