/*************************************************************************************************
*                                                                                                *
* Questo file contiene tutte le costanti necessarie per lo sviluppo dei plug-ins VST compatibili *
*                                                                                                *
*************************************************************************************************/


#ifndef __inc_VstConst__
#define __inc_VstConst__

/* Dato che gli sviluppatori Steinberg assumono che le costanti letterali (long = 'abcd')
   siano nell'ordine specificato da Microsoft Visual C++,  necessario definire manualmente
   l'ordine dei caratteri in una costante long per utilizzarle in altri compilatori.
*/
#define makefour(a,b,c,d) ((unsigned)(d) | \
                          ((unsigned)(c)<< 8) | \
		                  ((unsigned)(b) << 16) | \
                          ((unsigned)(a) << 24 ))

/* Definiamo alcuni tipi che ci torneranno utili in seguito durante gli algoritmi DSP */
typedef float     Sample;
typedef float*   pSample;
typedef float** ppSample;


#define kEffectMagic makefour('V','s','t','P') //'VstP'


/* Flag relativi all'effetto */

#define effFlagsHasEditor		1	/* Specifica se l'effetto ha un editor grafico */
#define effFlagsHasClip			2	/* Attiva il led "clip" nell'unit a rack. Non funzionante */
#define effFlagsHasVu			4	/* Ritorna il livello per i Vu-Meter. Non funzionante */
#define effFlagsCanMono			8	/* Ha senso un input monofonico su un effetto stereo */
#define effFlagsCanReplacing	16	/* Puo' processare rimpiazzandoi dati sul bus di out */
#define effFlagsProgramChunks	32	/* I dati dei programmi sono manipolati in formato binario */

/* Messaggi ricevuti dall'host, parte dello Steinberg VST SDK */

enum
{
	effOpen = 0,		// initialise
	effClose,			// exit, release all memory and other resources!

	effSetProgram,		// program no in <value>
	effGetProgram,		// return current program no.
	effSetProgramName,	// user changed program name (max 24 char + 0) to as passed in string
	effGetProgramName,	// stuff program name (max 24 char + 0) into string

	effGetParamLabel,	// stuff parameter <index> label (max 8 char + 0) into string
						// (examples: sec, dB, type)
	effGetParamDisplay,	// stuff parameter <index> textual representation into string
						// (examples: 0.5, -3, PLATE)
	effGetParamName,	// stuff parameter <index> label (max 8 char + 0) into string
						// (examples: Time, Gain, RoomType)
	effGetVu,			// called if (flags & (effFlagsHasClip | effFlagsHasVu))

	// system

	effSetSampleRate,	// in opt (float)
	effSetBlockSize,	// in value
	effMainsChanged,	// the user has switched the 'power on' button to
						// value (0 off, else on). This only switches audio
						// processing; you should flush delay buffers etc.
	// editor

	effEditGetRect,		// stuff rect (top, left, bottom, right) into ptr
	effEditOpen,		// system dependant Window pointer in ptr
	effEditClose,		// no arguments
	effEditDraw,		// draw method, ptr points to rect
	effEditMouse,		// index: x, value: y
	effEditKey,			// system keycode in value
	effEditIdle,		// no arguments. Be gentle!
	effEditTop,			// window has topped, no arguments
	effEditSleep,		// window goes to background

	// new

	effIdentify,		// returns 'NvEf'
	effGetChunk,		// host requests pointer to chunk into (void**)ptr, byteSize returned
	effSetChunk,		// plug-in receives saved chunk, byteSize passed
	effNumOpcodes
};

//---------------------------------------------------------------------------------------------
// audioMaster opCodes
//---------------------------------------------------------------------------------------------

enum
{
	audioMasterAutomate = 0,		// index, value, returns 0
	audioMasterVersion,				// vst version, currently 2 (0 for older)
	audioMasterCurrentId,			// returns the unique id of a plug that's currently
									// loading
	audioMasterIdle,				// call application idle routine (this will
									// call effEditIdle for all open editors too)
	audioMasterPinConnected,		// inquire if an input or output is beeing connected;
									// index enumerates input or output counting from zero,
									// value is 0 for input and != 0 otherwise. note: the
									// return value is 0 for <true> such that older versions
									// will always return true.
	audioMasterTempoAt = audioMasterPinConnected+6,

};

#endif	// __AEffect__
