#if !defined AudioFx
#define AudioFx AudioFx

#include "vstconst.h"
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/*******************************************************************************/
enum
{
 kNumDelays = 6,

 kRoomSize = 0, 
 kFeedback,
 kDamp,
 kDry, 
 kWet,

 kNumParam,
 kNumProgram = 32
};

struct Program_tag
{	
	float feedback;
	float room_size;
	float damp;
	float dry, wet;

	char name[32];
} typedef Program;


struct Delay_tag
{
	long int1; /* integrity check */

	float *samples, /* puntatore al buffer di campioni */
		  *prw,     /* puntatore lettura/scrittura al buffer */
		  *pmax;    /* puntatore al campione che indica la fine del buffer */
	float feedback, /* livello di feedback del delay */
		  level;    /* livello di output del delay */

	float coef,
		  history;  /* filtro passa-basso 1 ordine */

	long  int2; /* integrity check */

} typedef Delay;


float Delay_tap (Delay* delay, float input); /* aggiorna il buffer di delay */
float Allpass_tap (Delay* delay, float input); /* aggiorna il buffer di delay */
void  Delay_init (Delay* delay, Program* program, float samplerate); /* inizializza i delay */
void  Delay_clear (Delay* delay); /* libera i delay dalla memoria */

/*******************************************************************************/

typedef	long (*audioMasterCallback)(struct AudioFx *effect, long opcode, long index,
		long value, void *ptr, float opt);

struct AudioFx
{
	/* Membri VST che Cubase e Wavelab condividono */
	long magic;			/* 'VstP' */
	long (*dispatcher)(struct AudioFx *effect,
		               long opCode,
					   long index,
					   long value,
					   void *ptr,
					   float opt);

	void (*process)(struct AudioFx *effect,
		            float **inputs,
					float **outputs,
					long sampleframes);
	void (*setParameter)(struct AudioFx *effect, long index, float parameter);
	float (*getParameter)(struct AudioFx *effect, long index);

	long numPrograms;   /* Numero di programmi */
	long numParams;		/* Numero di parametri (per programma) */
	long numInputs;		/* Numero di canali audio "virtuali" in ingresso */
	long numOutputs;	/* Numero di canali in uscita */
	long flags;			/* Flag delle caratteristiche dell'effetto */
	long resvd1;		/* riservato (deve essere 0) */
	long resvd2;		/* riservato (deve essere 0) */
	long initialDelay;	/* Per gli algoritmi che hanno bisogno di conoscere in anticipo i campioni. */
	long realQualities;	/* Qualit in tempo reale */
	long offQualities;	/* Qualit non in tempo reale */
	float ioRatio;		/* Rapporto I/O del tasso di campionamento (Non implementato) */
	void *object;		/* Punta all'oggetto-classe */
	void *user;			/* Riservato all'utente */
	long uniqueID;		/* Identificatore unico del plug-in */

	long version;		/* 1 */
	void (*processReplacing)(struct AudioFx *effect,
		                     float **inputs,
							 float **outputs,
							 long sampleframes);
	char future[60];	// Riservato per fututi utilizzi (impostare a zero)
	/* Fine struttura VST */


	/* Dati relativi al plug-in che tornano spesso utili... */
	audioMasterCallback master;
	float sampleRate;
	long blockSize;
	long nCurProgram;
	long cBlockSize;    

	Program programs[kNumProgram];
	
	struct Delay_tag dly_left[6]; /* delay a sinistra */
	struct Delay_tag aps_left[3]; /* allpass a sinistra */

	struct Delay_tag dly_right[6]; /* delay a destra */
	struct Delay_tag aps_right[3];
	float dry, 
		  wet;

	/* Fine struttura */
};


#define FX struct AudioFx* fx

#if defined __cplusplus
extern "C" {
#endif

	// Processing function
	void Process (FX, ppSample ppInput, ppSample ppOutput, long nSamples);
	void ProcessReplacing (FX, ppSample ppInput, ppSample ppOutput, long nSamples);

	// Get/Set parameter value
	void SetParameter (FX, long nParameter, float value);
	float GetParameter (FX, long nParameter);

	// Get/Set program
	long GetProgram (FX);
	void SetProgram (FX, long nProgram);

	// Get/Set program name
	void SetProgramName (FX, /* const */ char* szProgramName);
	void GetProgramName (FX, char* szProgramName);

	// Get display text
	void GetParameterLabel (FX, long nParam, char* szText);
	void GetParameterDisplay (FX, long nParam, char* szText);
	void GetParameterName (FX, long nParam, char* szText);

	// Get VU-Meter levels (unsupported in Cubase)
	Sample GetVu (FX);


	long GetChunk (FX, void** ppData, long isPreset);
	long SetChunk(FX, void* data, long byteSize, long isPreset);

	// Get/Set sample rate.
	void SetSampleRate(FX, float sampleRate);
	void SetBlockSize(FX, long blockSize);

	// Suspend/Resume
	void Suspend(FX);
	void Resume(FX);

	struct AudioFx* CreateEffect(audioMasterCallback amc);
	void DestroyEffect(struct AudioFx* fx);
    long MessageDispatcher (struct AudioFx *e,
		                    long opCode,
							long index,
							long value,
							void *ptr,
							float opt);




#if defined __cplusplus
}
#endif

#endif
