#include "audiofx.h"

void imposta_delay (struct Delay_tag* delay, float ms, float fb, float sr, float level)
{
	int nsamples;

	delay->int1 = 'int1';
	delay->int2 = 'int2';

	if (ms > 250) ms = 250;
	if (ms < 1) ms = 1;
			
	nsamples = ms * sr / 1000.0F;

	/* imposta feedback */
	delay->feedback = fb;

	/* imposta lunghezza delay */
	delay->pmax = delay->samples + nsamples;

	/* reset puntatore lettura/scrittura */
	delay->prw = delay->samples;

	/* azzera buffer di delay */
	memset(delay->samples, 0, nsamples*sizeof(float));

	/* imposta livello */
	delay->level = level;	
}

void imposta_delay_lpf (struct Delay_tag* delay, float ms, float fb, float sr, float level, float lpf)
{
	imposta_delay(delay, ms, fb, sr, level);

	delay->coef = lpf;
	delay->history = 0;	
}



void update (struct AudioFx* fx)
{
	int c;
	float ms_delay;
	float decorr = 1.02; /* decorrelazione sinistro/destro */
	struct Program_tag* pg = fx->programs + fx->nCurProgram;
	float damp = pg->damp;
	

	/* delay dei filtri comb */
	
	ms_delay = 40 + pg->room_size * 40;
	imposta_delay_lpf(fx->dly_left, ms_delay, 0.9F * pg->feedback, fx->sampleRate, 0.16F, damp);
	imposta_delay_lpf(fx->dly_right, ms_delay * decorr, 0.9F * pg->feedback, fx->sampleRate, 0.16F, damp);

	/* 2 delay, da 30 a 55 ms... */
	ms_delay = 44 + pg->room_size * 44;
	imposta_delay_lpf(fx->dly_left + 1, ms_delay * decorr, 0.8F * pg->feedback, fx->sampleRate, 0.16F, damp);
	imposta_delay_lpf(fx->dly_right + 1, ms_delay, 0.8F * pg->feedback, fx->sampleRate, 0.16F, damp);

	/* 3 delay, da 47 a 80 ms... */
	ms_delay = 53 + pg->room_size * 53;
	imposta_delay_lpf(fx->dly_left + 2, ms_delay, 0.7F * pg->feedback, fx->sampleRate, 0.16F, damp);
	imposta_delay_lpf(fx->dly_right + 2, ms_delay * decorr, 0.7F * pg->feedback, fx->sampleRate, 0.16F, damp);

	/* 4 delay, da 80 a 120 ms... */
	ms_delay = 57 + pg->room_size * 57;
	imposta_delay_lpf(fx->dly_left + 3, ms_delay * decorr, 0.6F * pg->feedback, fx->sampleRate, 0.16F, damp);
	imposta_delay_lpf(fx->dly_right + 3, ms_delay, 0.6F * pg->feedback, fx->sampleRate, 0.16F, damp);

	/* 5 delay, da 50 a 90 ms... */
	ms_delay = 63 + pg->room_size * 63;
	imposta_delay_lpf(fx->dly_left + 4, ms_delay * decorr, 0.5F * pg->feedback, fx->sampleRate, 0.16F, damp);
	imposta_delay_lpf(fx->dly_right + 4, ms_delay, 0.5F * pg->feedback, fx->sampleRate, 0.16F, damp);

	/* 6 delay, da 15 a 25 ms... */
	ms_delay = 67 + pg->room_size * 67;
	imposta_delay_lpf(fx->dly_left + 5, ms_delay * decorr, 0.4F * pg->feedback, fx->sampleRate, 0.16F, damp);
	imposta_delay_lpf(fx->dly_right + 5, ms_delay, 0.4F * pg->feedback, fx->sampleRate, 0.16F, damp);

	/* allpass 1 */
	imposta_delay(fx->aps_left, 7 * decorr, 0.92F, fx->sampleRate, 1);
	imposta_delay(fx->aps_right, 7, 0.92F, fx->sampleRate, 1);

	/* allpass 2 */
	imposta_delay(fx->aps_left + 1, 5, 0.97F, fx->sampleRate, 1);
	imposta_delay(fx->aps_right + 1, 5 * decorr, 0.97F, fx->sampleRate, 1);
	
	/* allpass 3 */
	imposta_delay(fx->aps_left + 2, 11, 0.87F, fx->sampleRate, 1);
	imposta_delay(fx->aps_right + 2, 11 * decorr, 0.87F, fx->sampleRate, 1);

	fx->dry = pg->dry;
	fx->wet = pg->wet;
}



/* Imposta il valore del parametro nParameter. */
void SetParameter (struct AudioFx* fx, long nParameter, float value)
{
	struct Program_tag *pg = &fx->programs[fx->nCurProgram];

	switch (nParameter)
	{
		case kRoomSize: pg->room_size = value; break;
		case kFeedback: pg->feedback = value; break;
		case kWet: pg->wet = value; break;
		case kDry: pg->dry = value; break;
		case kDamp: pg->damp = value; break;
	}

	update(fx); /* Aggiorna! */
}

/* Imposta il numero di programma corrente. */
void SetProgram (struct AudioFx* fx, long nProgram)
{
	fx->nCurProgram = nProgram;
	update(fx);
}

/* Imposta il nome del programma corrente. */
void SetProgramName (struct AudioFx* fx, char* szProgramName)
{
	strcpy(fx->programs[fx->nCurProgram].name, szProgramName);
}

/* Imposta la frequenza di campionamento */
void SetSampleRate (struct AudioFx* fx, float sampleRate)
{
	int samples = sampleRate / 4;

	if (sampleRate != fx->sampleRate || fx->dly_left[0].samples == 0)
	{
		/* libera i buffer */
		if (fx->dly_left[0].samples) free(fx->dly_left[0].samples);
		if (fx->dly_left[1].samples) free(fx->dly_left[1].samples);
		if (fx->dly_left[2].samples) free(fx->dly_left[2].samples);
		if (fx->dly_left[3].samples) free(fx->dly_left[3].samples);
		if (fx->dly_left[4].samples) free(fx->dly_left[4].samples);
		if (fx->dly_left[5].samples) free(fx->dly_left[5].samples);

		if (fx->dly_right[0].samples) free(fx->dly_right[0].samples);
		if (fx->dly_right[1].samples) free(fx->dly_right[1].samples);
		if (fx->dly_right[2].samples) free(fx->dly_right[2].samples);
		if (fx->dly_right[3].samples) free(fx->dly_right[3].samples);
		if (fx->dly_right[4].samples) free(fx->dly_right[4].samples);
		if (fx->dly_right[5].samples) free(fx->dly_right[5].samples);
		
		if (fx->aps_left[0].samples) free(fx->aps_left[0].samples);
		if (fx->aps_left[1].samples) free(fx->aps_left[1].samples);
		if (fx->aps_left[2].samples) free(fx->aps_left[2].samples);
		
		if (fx->aps_right[0].samples) free(fx->aps_right[0].samples);
		if (fx->aps_right[1].samples) free(fx->aps_right[1].samples);
		if (fx->aps_right[2].samples) free(fx->aps_right[2].samples);

		/* alloca i buffer */
		fx->dly_left[0].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_left[0].samples, 0, samples*sizeof(float));

		fx->dly_left[1].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_left[1].samples, 0, samples*sizeof(float));

		fx->dly_left[2].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_left[2].samples, 0, samples*sizeof(float));

		fx->dly_left[3].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_left[3].samples, 0, samples*sizeof(float));

		fx->dly_left[4].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_left[4].samples, 0, samples*sizeof(float));
		
		fx->dly_left[5].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_left[5].samples, 0, samples*sizeof(float));

		
		fx->dly_right[0].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_right[0].samples, 0, samples*sizeof(float));
		
		fx->dly_right[1].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_right[1].samples, 0, samples*sizeof(float));
		
		fx->dly_right[2].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_right[2].samples, 0, samples*sizeof(float));
		
		fx->dly_right[3].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_right[3].samples, 0, samples*sizeof(float));
		
		fx->dly_right[4].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_right[4].samples, 0, samples*sizeof(float));
		
		fx->dly_right[5].samples = malloc (sizeof(float)*samples);
		memset(fx->dly_right[5].samples, 0, samples*sizeof(float));
		


		fx->aps_left[0].samples = malloc (sizeof(float)*samples);
		memset(fx->aps_left[0].samples, 0, samples*sizeof(float));
		
		fx->aps_left[1].samples = malloc (sizeof(float)*samples);
		memset(fx->aps_left[1].samples, 0, samples*sizeof(float));

		fx->aps_left[2].samples = malloc (sizeof(float)*samples);
		memset(fx->aps_left[2].samples, 0, samples*sizeof(float));
		
		fx->aps_right[0].samples = malloc (sizeof(float)*samples);
		memset(fx->aps_right[0].samples, 0, samples*sizeof(float));
		
		fx->aps_right[1].samples = malloc (sizeof(float)*samples);
		memset(fx->aps_right[1].samples, 0, samples*sizeof(float));		

		fx->aps_right[2].samples = malloc (sizeof(float)*samples);
		memset(fx->aps_right[2].samples, 0, samples*sizeof(float));		

		/* aggiorna l'effetto */
		update(fx);		
	}
}

/* Chiamata quando l'effetto viene spento */
void Suspend (struct AudioFx* fx)
{
	/* Azzera le linee di ritardo! */
	int k;
	for (k = 0; k < 6; ++k)
	{	
		memset (fx->dly_left[k].samples, 0, (int)(sizeof(float)*fx->sampleRate/4));
		memset (fx->dly_right[k].samples, 0, (int)(sizeof(float)*fx->sampleRate/4));
		if (k < 3)
		{
			memset(fx->aps_left[k].samples, 0, (int)(sizeof(float)*fx->sampleRate/4));
			memset(fx->aps_right[k].samples, 0, (int)(sizeof(float)*fx->sampleRate/4));
		}
	}
}

/* Chiamata quando l'effetto viene acceso */
void Resume (struct AudioFx* fx)
{

}
